//    WeIRCd administration tool
//    Copyright (C) 2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "admintool.h"

struct localinstallinfo* iinfo = NULL;

void localinstall_CreateInfo() {
	if (iinfo == NULL) {
		iinfo = malloc(sizeof(struct localinstallinfo));
		memset(iinfo,0,sizeof(struct localinstallinfo));
	}
}
void localinstall_SetBinPath(const char* path) {
	localinstall_CreateInfo();
	if (iinfo->binpath != NULL) {free(iinfo->binpath);iinfo->binpath = NULL;}
	if (path == NULL) {return;}
	iinfo->binpath = malloc(strlen(path)+1);strcpy(iinfo->binpath,path);
}
void localinstall_SetDataDir(const char* path) {
	localinstall_CreateInfo();
	if (iinfo->datadir != NULL) {free(iinfo->datadir);iinfo->datadir = NULL;}
	if (path == NULL) {return;}
	iinfo->datadir = malloc(strlen(path)+1);strcpy(iinfo->datadir,path);
}
char staticemptybuf[] = "";
const char* localinstall_GetDataDir() {
	if (iinfo == NULL) {return staticemptybuf;}
	if (iinfo->datadir != NULL) {
		return iinfo->datadir;
	}else{
		return staticemptybuf;
	}
}
const char* localinstall_GetBinPath() {
	if (iinfo == NULL) {return staticemptybuf;}
	if (iinfo->binpath != NULL) {
		return iinfo->binpath;
	}else{
		return staticemptybuf;
	}
}

void DirWithSlash(char* dir, int maxlen) {
	if (strendswith(dir,"/") != 0 && strendswith(dir,"\\") != 0) {
		int slash = 0;
		if (charpos(dir,'\\') >= 0 && (charpos(dir,'\\') < charpos(dir,'/') || charpos(dir,'/') < 0)) {
			slash = 1;
		}
		if (slash == 0) {
			stradd(dir,"/",maxlen);
		}else{
			stradd(dir,"\\",maxlen);
		}
	}
}
#ifdef WIN
int localinstall_FindWin() {
	char value[101];
	LONG retvalue;
	DWORD type;
	HKEY hkey;
	DWORD len=100;
	int i = 0;
	retvalue = RegOpenKeyEx (HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\WeIRCd", 0L, KEY_READ , &hkey);
	if (retvalue == ERROR_SUCCESS) {
		retvalue = RegQueryValueEx(hkey, "InstallLocation", NULL, &type,(LPBYTE)&value, &len);
		if (retvalue != ERROR_SUCCESS) {
			RegCloseKey(hkey);return -1;
		}
		if (type != REG_SZ && type != REG_EXPAND_SZ) {
			RegCloseKey(hkey);return -1;
		}
		char binpath[500];
		strncpy(binpath,value,499);binpath[499] = 0;
		DirWithSlash(binpath,499);
		localinstall_SetDataDir(binpath);
		stradd(binpath,"WeIRCd.exe",499);
		localinstall_SetBinPath(binpath);
		RegCloseKey(hkey);
	}else{
		RegCloseKey(hkey);
		return -1;
	}
	return 0;
}
#endif
int localinstall_Find() {
#ifdef WIN
	return localinstall_FindWin();
#else
#ifdef UNIXLOCALHACK
        localinstall_SetBinPath("/usr/bin/weircd/");
        localinstall_SetDataDir("/usr/local/var/weircd/");
        return 0;
#endif
	return -1;
#endif
}
int localinstall_IsServerRunning(int* writepid) {
#ifdef WIN
	//try to find process
	DWORD aProcesses[1024], cbNeeded;
	unsigned int cProcesses;
	if (!EnumProcesses(aProcesses, sizeof(aProcesses), &cbNeeded)) {
		return;
	}
	// Calculate how many process identifiers were returned.
	cProcesses = cbNeeded / sizeof(DWORD);
	// Print the name and process identifier for each process.
	unsigned int i = 0;
	while (i < cProcesses) {
		HANDLE proc = OpenProcess(PROCESS_QUERY_INFORMATION |
                                   PROCESS_VM_READ,
                                   FALSE, aProcesses[i]);
		TCHAR pname[MAX_PATH] = TEXT("");
		if (proc != NULL) {
			HMODULE hMod;
			DWORD cbNeeded2;
			if (EnumProcessModules(proc, &hMod, sizeof(hMod), &cbNeeded2)) {
				GetModuleBaseName(proc, hMod, pname, 
				sizeof(pname)/sizeof(TCHAR));
			}
		}
		if (strcasecmp(pname,"weircd.exe") == 0) {
			*writepid = aProcesses[i];
			return 0;
		}
		i++;
	}
	return -1;
#else
        return -1;
#endif
}
char processstarterror[500] = "";
const char* localinstall_GetProcessStartError() {
	return processstarterror;
}
#ifdef WIN
char winerrbuf[500] = "";
const char* winerr(int id) {
	if (id < 0) {
		id = GetLastError();
	}
	LPTSTR Error = 0;
	if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                    0,
                    id,
                    0,
                    (LPTSTR)&winerrbuf,
                    499,
                    NULL) == 0) {
		strcpy(winerrbuf,"<Failed to translate error code>");
	}
	winerrbuf[499] = 0;
	return winerrbuf;
}
#endif
#ifdef WIN
int localinstallwin_ReadyStartupAndPipe(HANDLE* stdout_writeto,HANDLE* stdout_readfrom,STARTUPINFO* si, PROCESS_INFORMATION* pi) {
	//clear structs
	memset(si, 0, sizeof(STARTUPINFO));
	si->cb = sizeof(si);
	memset(pi, 0, sizeof(PROCESS_INFORMATION));
	//create pipe
	SECURITY_ATTRIBUTES secattr;
	memset(&secattr,0,sizeof(secattr));
	secattr.nLength = sizeof(secattr);
	secattr.lpSecurityDescriptor = NULL;
	secattr.bInheritHandle = TRUE; 
	if (!CreatePipe(stdout_readfrom,stdout_writeto,&secattr,0)) {
		return -1;
	}
	//set pipe into startup info
	
	GetStartupInfo(si);
	si->hStdOutput = *stdout_writeto;
	si->hStdError = *stdout_writeto;
	si->hStdInput = 0;
	si->dwFlags = si->dwFlags|STARTF_USESTDHANDLES; //use our pipes
}
#endif
#ifdef WIN
int localinstallwin_StartLocalWeIRCd(STARTUPINFO* si, PROCESS_INFORMATION* pi, int getversion) {
	//construct full path as a quoted string
	char execute[500];
	char getv[] = " --version";
	if (getversion == 0) {strcpy(getv,"");}
	snprintf(execute,499,"\"%s\"%s",localinstall_GetBinPath(),getv);execute[499] = 0;
	printf(">>%s<<\n",execute);
	/*if (strlen(execute) >= 499) {
		execute[489] = '\"';
	}*/
	//this is rather huge and inconvenient. but meh, what can we do
	if (!CreateProcess(NULL,execute,
	NULL, // Process handle not inheritable
	NULL, // Thread handle not inheritable
	TRUE, // Set handle inheritance to TRUE !
	DETACHED_PROCESS, // No creation flags
	NULL,  // Use parent's environment block
	localinstall_GetDataDir(), // Use parent's starting directory 
	si, // Pointer to STARTUPINFO structure
	pi ) // Pointer to PROCESS_INFORMATION structure
	) {
		printf("couldn't create process :<\n");
		return -1;
	}
	printf("process created...\n");
	return pi->dwProcessId;
}
#endif
char staticversionbuf[50];
const char* localinstall_GetVersion() {
	/* startup weircd with --version and parse the version returned */
#ifdef WIN
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	HANDLE stdout_writeto,stdout_readfrom;
	//setup pipe
	if (localinstallwin_ReadyStartupAndPipe(&stdout_writeto,&stdout_readfrom,&si,&pi) < 0) {
		return NULL;
	}
	int pid = localinstallwin_StartLocalWeIRCd(&si,&pi,1);
	if (pid < 0) {
		return NULL;
	}
	//can we close them? probably. let's try.
	CloseHandle( pi.hProcess );
	CloseHandle( pi.hThread );
	char mbuf[1024];int bread = 0;int avail;int bytesleft;
	long starttime = time(NULL);
	while (bread == 0) {
		//wait for data
		if (!PeekNamedPipe(stdout_readfrom,mbuf,1023,(DWORD*)&bread,(DWORD*)&avail,(DWORD*)&bytesleft)) {
			break;
		}else{
			if (starttime +1 < time(NULL)) {break;}
		}
	}
	int r;
	if (bread >= 0 && bread <= 1023) {
		mbuf[bread] = 0;
	}
	if (strstw(mbuf,"weircd") != 0) {
		return NULL;
	}
	cut(mbuf,strlen("weircd"));
	if (charpos(mbuf,'-') <= 3) {return NULL;}
	mbuf[charpos(mbuf,'-')] = 0;
	if ((mbuf[0] > '9' || mbuf[0] < '0')) {return NULL;}
	strncpy(staticversionbuf,mbuf,49);staticversionbuf[49] = 0;
	CloseHandle(stdout_readfrom);
	CloseHandle(stdout_writeto);
	//CloseHandle( pi.hProcess );
	//CloseHandle( pi.hThread );
	return staticversionbuf;
#else
        return NULL;
#endif
}
int localinstall_Run(PIPEHANDLE* readfrompipe, PIPEHANDLE* writetopipe) {
#ifdef WIN
	strcpy(processstarterror,"");
	//startup info and process info structs that will be filled
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	HANDLE stdout_writeto,stdout_readfrom;
	//setup pipe
	if (localinstallwin_ReadyStartupAndPipe(&stdout_writeto,&stdout_readfrom,&si,&pi) < 0) {
		strncpy(processstarterror,winerr(-1),499);processstarterror[499] = 0;
		CloseHandle(stdout_readfrom);
		CloseHandle(stdout_writeto);
		return;
	}
	int pid = localinstallwin_StartLocalWeIRCd(&si,&pi,0);
	if (pid < 0) {
		strncpy(processstarterror,winerr(-1),499);processstarterror[499] = 0;
		char nr[10];snprintf(nr,9," (%d)",GetLastError());nr[9] = 0;
		stradd(processstarterror,nr,499);
		return -1;
	}
	//can we close them? probably. let's try.
	CloseHandle( pi.hProcess );
	CloseHandle( pi.hThread );
	*readfrompipe = stdout_readfrom;
	*writetopipe = stdout_writeto;
	return pid;
#else
	strcpy(processstarterror,"Not implemented yet");
	return -1;
#endif
}
void localinstall_ClosePipes(PIPEHANDLE readfrompipe, PIPEHANDLE writetopipe) {
#ifdef WIN
	CloseHandle(readfrompipe);
	CloseHandle(writetopipe);
#endif
}
char retbuf[1024];
#ifdef WIN
void localinstall_CheckPipeWin(PIPEHANDLE readfrompipe, void* parseline, void* userdata) {
	void (*p)(const char*, void*) = parseline;
	char buf[1024];
	DWORD bread = 0;DWORD avail = 0;
	if (!PeekNamedPipe(readfrompipe,buf,1023,&bread,&avail,NULL)) {
		return;
	}
	//check to see if there is any data to read from stdout
	if (bread != 0) {
		memset(&buf,0,sizeof(buf));
		while (avail > 0) {
			ReadFile(readfrompipe,buf,1023,&bread,NULL); //read the stdout pipe
			if (bread <= 0) {break;}
			buf[bread] = 0;
			//printf("read: %d\n",bread);
			p(buf, userdata);
			memset(&buf,0,sizeof(buf));
			if (avail > bread) {
				avail -= bread;
			}else{
				break;
			}
		}
	}
	return;
}
#endif
void localinstall_CheckPipe(PIPEHANDLE readfrompipe, void* parseline, void* userdata) {
#ifdef WIN
	localinstall_CheckPipeWin(readfrompipe, parseline, userdata);
#endif
#ifndef WIN

#endif
}
#ifdef WIN
void localinstall_SendWinSig(int pid, int sig) {
	HANDLE hThreadSnap = INVALID_HANDLE_VALUE; 
	THREADENTRY32 te32; 
	// Take a snapshot of all running threads  
	hThreadSnap = CreateToolhelp32Snapshot( TH32CS_SNAPTHREAD, 0 ); 
	if (hThreadSnap == INVALID_HANDLE_VALUE) {return;}
	// Fill in the size of the structure before using it. 
	te32.dwSize = sizeof(THREADENTRY32); 
	// Retrieve information about the first thread,
	// and exit if unsuccessful
	if (!Thread32First( hThreadSnap, &te32 ))  {
		printf("error! (1)\n");
		//printError( "Thread32First" );  // Show cause of failure
		CloseHandle( hThreadSnap );     // Must clean up the snapshot object!
		return;
	}

	// Now walk the thread list of the system,
	// and display information about each thread
	// associated with the specified process
	int tid = -1;
	do { 
		if (te32.th32OwnerProcessID == pid) {
			tid = te32.th32ThreadID;
			break;
		}
	} while( Thread32Next(hThreadSnap, &te32 ) ); 
	if (tid > 0) {
		if (!PostThreadMessage(tid, sig, 0, 0)) {
			printf("fail: %s (%d)\n",winerr(-1),localpid);
		}else{
			printf("signal %d sent\n",sig-WM_APP);
		}
	}else{
		printf("error! (2)\n");
	}
	//  Don't forget to clean up the snapshot object.
	CloseHandle( hThreadSnap );
	return;
}
#endif
void localinstall_RehashSig(int pid) {
#ifdef WIN
	localinstall_SendWinSig(pid, WM_APP+3);
#endif
}
void localinstall_TermSig(int pid) {
#ifdef WIN
	localinstall_SendWinSig(pid, WM_APP+2);
#endif
}
int status_currentlocalusers;
int status_currentglobalusers;
int status_opers;
#ifdef WIN
int localinstall_ReadStatusWin() {
	char pathbuf[100];
	if (GetTempPath(99,pathbuf) <= 0) {return -1;}
	char tg[150];
	strcopy(tg,pathbuf,149);
	if (strendswith(tg,"/") != 0) {
		stradd(tg,"/",149);
	}
	stradd(tg,"weircd-status",149);
	FILE* r = fopen(tg,"rb");
	if (!r) {return -1;}
	status_currentlocalusers = 0;
	status_currentglobalusers = 0;
	status_opers = 0;
	char line[300];
	while (fgets(line,299,r) != NULL) {
		if (strstw(line,"users-local-current ") == 0) {
			cut(line,strlen("users-local-current "));
			trimleft(line);cutonfirstspace(line);
			status_currentlocalusers = atoi(line);
			continue;
		}
		if (strstw(line,"users-global-current ") == 0) {
			cut(line,strlen("users-global-current "));
			trimleft(line);cutonfirstspace(line);
			status_currentglobalusers = atoi(line);
			continue;
		}
		if (strstw(line,"opers ") == 0) {
			cut(line,strlen("opers "));
			trimleft(line);cutonfirstspace(line);
			status_opers = atoi(line);
			continue;
		}
	}
	fclose(r);
	return 0;
}
#endif
int localinstall_ReadStatus() {
#ifdef WIN
	return localinstall_ReadStatusWin();
#endif
#ifndef WIN
	return -1;
#endif
}
