//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

void splitcmdargs(const char* cmd, int maxargs, int argbuflen, int lastargbuflen, char** argbufs, char* lastargbuf, int* finalargcount, char** finalargpointers) {
	/* Separate the long last parameter and the "normal" parameters in front of it: */
	char* remainingparams = alloca(strlen(cmd)+1); //temporary buffer for the "normal" parameters
	int colonpos = charpos(cmd,':'); //get the position of the seperating clon
	if (colonpos >= 0) {
		strcopy(lastargbuf,cmd+colonpos+1,lastargbuflen-1);
		strcpy(remainingparams,cmd);
		remainingparams[colonpos] = 0;
		if (onlyspaces(lastargbuf) == 0) {
			strcpy(lastargbuf,"");
		}
	}else{
		strcpy(remainingparams,cmd);
		strcpy(lastargbuf,"");
	}
	/* Now cycle through the "normal" parameters and split them up and place them into our buffers: */
	int paramcount = 0;
	int r = 0;
	int k = charpos(remainingparams+r,' ');
	while (k >= 0) { /* we have space chars. we need to break up at every single space char */
		if (k == 0) {
			r++; /* empty param, skip */
		}else{
			int copybytes = argbuflen-1;
			if (k < copybytes) {copybytes = k;} /* only copy the bytes in front of the space char */
			strcopy(argbufs[paramcount], remainingparams+r, copybytes); /* copy parameter */
			paramcount++;
			r += k+1; /* cut space chars from remaining commands list */
			if (paramcount == maxargs-1) {break;} /* limit reached - WARNING: always leave last parameter free for big long last parameter */
		}
		k = charpos(remainingparams+r,' ');
	}
	if (strlen(remainingparams+r) > 0 && paramcount < maxargs-1) { /* we have a parameter remaining and not yet reached the limit - WARNING: always leave last parameter free (see above)  */
		strcopy(argbufs[paramcount], remainingparams+r, argbuflen-1);
		paramcount++;
	}
	/* Now prepare our nice pointers so we can just pass */
	int finalcount = 0; //final parameter count including long last param
	r = 0;
	while (r < paramcount) { /* cycle our "normal" params first */
		finalargpointers[r] = argbufs[r];
		r++;
	}
	finalcount += paramcount;
	if (strlen(lastargbuf) > 0) { /* see if we also have a long last param */
		finalargpointers[r] = lastargbuf;r++;
		finalcount++;
	}
	*finalargcount = finalcount; /* set the final count to the given pointer */
	return; /* done! */
}
void obtainsplitargsbuf(char*** ptrfield, int maxargs, int arglen) {
	*ptrfield = malloc(sizeof(void*)*maxargs);
	int r = 0;
	while (r < maxargs) {
		(*ptrfield)[r] = malloc(arglen);
		r++;
	}
}
void obtainptrbuf(char*** ptrfield, int maxargs) {
	*ptrfield = malloc(sizeof(void*)*maxargs);
}