//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"


void* listoflinks = NULL; //our link list.

void links_FreeStructData(struct weircdlink* l) {
	if (l->interface != NULL) {
		free(l->interface);
	}
	if (l->protmodule != NULL) {
		free(l->protmodule);
	}
	if (l->sendpassword != NULL) {
		free(l->recvpassword);
	}
	if (l->sendpassword != NULL) {
		free(l->recvpassword);
	}
}
void links_FreeListEntry(void* entry) { /* free a single entry in the listoflinks or listofwantedlinks (but do not remove it, will just empty it) */
	struct weircdlink* l = linkedlist_GetData(entry);
	links_FreeStructData(l);
	free(l);
	linkedlist_SetData(entry,NULL);
}
void* links_Add(int isactive, const char* interface, int port, int iptype, const char* protocol, const char* sendpassword, const char* recvpassword, void* dependson) {
	/* Add a new link struct to our linked list of links */
	struct weircdlink l;memset(&l,0,sizeof(l));
	if (isactive == 1) {
		l.type = LINKTYPE_ACTIVE;
	}else{
		l.type = LINKTYPE_PASSIVE;
	}
	l.iptype = iptype;
	l.port = port;
	l.activeconnectionid = -1;
	l.interface = malloc(strlen(interface)+1);
	if (dependson != NULL && isactive == 1) {
		l.dependson = dependson;
	}
	if (l.interface == NULL) {return NULL;}
	strcpy(l.interface,interface);
	l.protmodule = malloc(strlen(protocol)+1);
	if (l.protmodule == NULL) {
		links_FreeStructData(&l);return NULL;
	}
	strcpy(l.protmodule,protocol);
	if (sendpassword != NULL && isactive == 1) {
		l.sendpassword = malloc(strlen(sendpassword)+1);
		if (l.sendpassword == NULL) {
			links_FreeStructData(&l);return NULL;
		}
		strcpy(l.sendpassword,sendpassword);
	}
	if (recvpassword != NULL && isactive == 1) {
		l.recvpassword = malloc(strlen(recvpassword)+1);
		if (l.recvpassword == NULL) {
			links_FreeStructData(&l);return NULL;
		}
		strcpy(l.recvpassword,recvpassword);
	}
	void* result = linkedlist_Append(&listoflinks,&l,sizeof(l));
	if (result == NULL) {
		links_FreeStructData(&l);return NULL;	
	}
	return result;
}
int links_Activate(const char* tg, int port) {
	/* First, obtain a new connection id. While doing that, also search the link structure that matches */
	void* linkptr = NULL;
	int id = 0;
	void* item = listoflinks;
	while (item != NULL) {
		struct weircdlink* l = linkedlist_GetData(item);
		if (l->activeconnectionid == id) {
			id = l->activeconnectionid+1;
		}
		if (l->interface != NULL) {
			if (strcasecmp(l->interface,tg) == 0 && ((l->port == port && l->type == LINKTYPE_PASSIVE) || (l->port != port && l->type == LINKTYPE_ACTIVE))) {
				linkptr = item;
			}
		}
		item = linkedlist_NextItem(item);
	}
	if (linkptr == NULL) {return -1;} //no matching link structure found
	struct weircdlink* l = linkedlist_GetData(linkptr);
	/* Ok, now we will start the respective linking module and pass it our new connection id */
	if (l->activeconnectionid >= 0) {return -1;} //already activated
	if (modlist_LoadForLinking(l->protmodule) != 0) {
		return -1;
	}
	l->activeconnectionid = id;
	int useipv6 = 0;
	if (l->iptype == IPTYPE_IPV6) {useipv6 = 1;}
	if (l->type == LINKTYPE_PASSIVE) {
		if (linkingapi_StartPassiveLink(l->protmodule, l->interface, l->port, useipv6, l->activeconnectionid) != 0) {
			l->activeconnectionid = -1;
			return -1;
		}
	}else{
		l->activeconnectionid = -1;
		return -1;
	}
	return 0;
}

int links_IsModuleNeeded(const char* name) { /*check if any link is using this module or if we may unload it */
	void* item = listoflinks;
	while (item != NULL) {
		struct weircdlink* l = linkedlist_GetData(item);
		if (strcasecmp(name,l->protmodule) == 0) {
			return 0;
		}
		item = linkedlist_NextItem(item);
	}
	return -1;
}
char cyclemodule[50];
int cyclecount;
void links_StartIDCycle(const char* modname) {
	strcopy(cyclemodule,modname,49);
	cyclecount = 1;
}
int links_DoIDCycle() {
	int i = 0;
	void* item = listoflinks;
	while (item != NULL) {
		struct weircdlink* wl = linkedlist_GetData(item);
		if (strcasecmp(wl->protmodule,cyclemodule) == 0 && wl->activeconnectionid >= 0) {
			i++;
			if (i == cyclecount) {
				cyclecount++;
				return wl->activeconnectionid;
			}
		}
		item = linkedlist_NextItem(item);
	}
	return -1;
}
void links_Broadcast(const char* str) {
	
}
void links_Forward(const char* str, int exceptlink) {
	
}
void links_SendByUID(const char* str, USER user) {
	
}
void links_SendBySID(const char* str, int serverlink) {
	
}