//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

struct remoteserver {
	char* displayname;
	char* name;
	char* description;
	void* data;
	void* serverchildnodes;
	int hopcount;
};
void* serverrootnode = NULL;

void server_FreeData(struct remoteserver* r) {
	if (r->displayname != NULL) {free(r->displayname);}
	if (r->name != NULL) {free(r->name);}
	if (r->description != NULL) {free(r->description);}
}
SERVER server_Add(const char* name, const char* sid, const char* description, SERVER parent) {
	struct remoteserver r;memset(&r,0,sizeof(r));
	r.displayname = malloc(strlen(name)+1);
	if (r.displayname == NULL) {
		return NOSERVER;
	}
	strcpy(r.displayname,name);
	if (sid != NULL) {
		r.name = malloc(strlen(sid)+1);
		if (r.name == NULL) {
			server_FreeData(&r);return NOSERVER;
		}
		strcpy(r.name,sid);
	}else{
		r.name = malloc(strlen(name)+1);
		if (r.name == NULL) {
			server_FreeData(&r);return NOSERVER;
		}
		strcpy(r.name,name);
	}
	if (description != NULL) {
		r.description = malloc(strlen(description)+1);
		if (r.description == NULL) {
			server_FreeData(&r);return NOSERVER;
		}
		strcpy(r.description,description);
	}
	r.hopcount = 1;
	if (parent != NULL) {
		r.hopcount = 1+*(int*)((struct remoteserver*)linkedlist_GetData(parent))->hopcount;
		struct remoteserver* pserver = linkedlist_GetData(parent);
		return linkedlist_Append(&pserver->serverchildnodes,(void**)&r,sizeof(r));
	}
	return linkedlist_Append(&serverrootnode,(void**)&r,sizeof(r));
}
SERVER server_GetByName_internal(const char* name, void* list) {
	void* item = list;
	while (item != NULL) {
		struct remoteserver* r = linkedlist_GetData(item);
		if (strcasecmp(r->name,name) == 0) {
			return item;
		}else{
			SERVER s = server_GetByName_internal(name,r->serverchildnodes);
			if (s != NOSERVER) {return s;}
		}
		item = linkedlist_NextItem(item);
	}
	return NOSERVER;
}
const char* server_GetName(SERVER serv) {
        if (serv == NOSERVER) {return NULL;}
        return ((struct remoteserver*)linkedlist_GetData(serv))->name;
}
SERVER server_GetByName(const char* name) {
	return server_GetByName_internal(name, serverrootnode);
}
void server_SetDataPtr(SERVER serv, void* data) {
	if (serv == NOSERVER) {return;}
	struct remoteserver* r = linkedlist_GetData(serv);
	r->data = data;
}
void* server_GetDataPtr(SERVER serv) {
	if (serv == NOSERVER) {return NULL;}
	return ((struct remoteserver*)linkedlist_GetData(serv))->data;
}
USER server_AddUser(SERVER serv, const char* nick, const char* userid, const char* modes) {
	USER u = usr_AddUser(-1,-1,NULL,0); /* get us a new user */
	struct remoteserver* r = linkedlist_GetData(serv);
	udat(u)->hopcount = r->hopcount;
	udat(u)->homeserver = serv;
	if (modes != NULL) {
		int i = 0;
		while (i < strlen(modes)) {
			if (modes[i] == 'o') {
				udat(u)->isircop = 1;
			}
			if (modes[i] == 'S') {
				udat(u)->mode_servicesbot = 1;
			}
			i++;
		}
	}
	strcopy(udat(u)->nickname, nick, COMPILEOPTION_MAXNICKLEN);
	strcopy(udat(u)->suserid, userid, COMPILEOPTION_MAXUSERIDLEN);
	udat(u)->pleaseremove = 0;
	udat(u)->authed = 1;
	return u;
}
void server_RemoveUser(SERVER serv, const char* nick, const char* reason) {
	USER u = usr_FindByNick(nick);
	if (u == NOUSER) {return;}
	if (udat(u)->pleaseremove == 1) {return;}
	udat(u)->quitreason = malloc(strlen(reason)+1);
	if (udat(u)->quitreason != NULL) {
		strcpy(udat(u)->quitreason,reason);
	}
	userQuit(u, USERQUIT_CUSTOMREMOTEQUIT);
}
void server_RemoveAllUsers_internal(SERVER serv, SERVER splittedserv) {
	if (splittedserv == NOSERVER) {splittedserv = serv;}
	if (serv == NOSERVER) {return;}
	struct remoteserver* r = linkedlist_GetData(serv);
	void* item = r->serverchildnodes;
	while (item != NULL) {
		server_RemoveAllUsers_internal(item,splittedserv);
		void* nextitem = linkedlist_NextItem(item);
		linkedlist_RemoveItem(&r->serverchildnodes, item); 
		item = nextitem;
	}
	const char* splitted_subject = ((struct remoteserver*)linkedlist_GetData(splittedserv))->name;
	const char* splitted_from = NULL;
	if (splittedserv != NOSERVER) {
		splitted_from = ((struct remoteserver*)linkedlist_GetData(splittedserv))->name;
	}else{
		splitted_from = svname(); //ourselves
	}
	char splitmsg[500];
	snprintf(splitmsg,499,"%s %s",splitted_from, splitted_subject);
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (udat(u)->hopcount > 0 && udat(u)->homeserver == serv) {
			// FIXME: this seems bad. we are allocating a lot of space here.
			udat(u)->quitreason = malloc(strlen(splitmsg)+1);
			if (udat(u)->quitreason != NULL) {
				strcpy(udat(u)->quitreason,splitmsg);
			}
			userQuit(u,USERQUIT_NETSPLIT);
		}
		u = usr_DoCycle();
	}
	//now remove the server
	server_FreeData(linkedlist_GetData(serv)); /* free all allocated sub data (name, description strings etc) */
	if (r->hopcount <= 1) { /* luckily, this integer is not free'd */
		linkedlist_RemoveItem(&serverrootnode,serv);
	}
}
void server_Remove(SERVER serv) {
	server_RemoveAllUsers_internal(serv,serv);
}