//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

// ----- This is the WeIRCd API for FakeUsers ----- //
  /* Use this to connect fake users to the server.
     They can be used for integrated services or
     bots of any kind.
     
     This api is available without prior activation.

     Fake users are just like a normal user
     connection on which you can send and receive
     as if being a normal user client.

  Function needed: void module_FakeUserRecv(int handle, const char* data)
     This will be called when any data line is
     available for one of your fake connections.
     It will always give you exactly one line
     you may process.
     
   SOME HINTS:
     - If you don't use the weircd services api to
     login your connection as a services bot, it
     will be also object to flood protection, kills
     etc. like any other user. */

int weircd_fakeuser_Create();
  /* Returns a fake user handle. Use it with the following functions
     to use your fake user. When this returns -1, the user could not
     be created (rare but better be prepared) */
   
int weircd_fakeuser_Receive(int handle, char* buf, int count);
  /* A non-blocking receive function that will put a maximum
     amount of 'count' bytes into the 'buf' received from
     the virtual connection of the fake user addressed by the
     handle.
     When returning -1, there was no data to receive and 'buf'
     remains unchanged.
     In case the function returns a number greater than 0,
     it represents the count of bytes actually written into 'buf'
     (which can be any amount up to the maximum 'count').
     If this returns -1, the fake user's virtual connection
     has been disconnected and the fake user should be either
     reconnected or disposed. */
   
int weircd_fakeuser_Send(int handle, const char* line);
  /* Use this to send an IRC command (null terminated string)
     on a fake user connection.
     Line break doesn't need to be put at the end.
     Returns -1 if sending failed (that only happens when
     the handle doesn't represent a valid connected user).*/

void weircd_fakeuser_Reconnect(int handle);
  /* Reconnect a previously disconnected fake user again */

void weircd_fakeuser_Free(int handle);
  /* Free and dispose a fake user. You can also safely use
     this when the fake user is still connected, the virtual
     connection will then simply be closed aswell. The handle
     is obviously invalid afterwards (until it is assigned
     to a newly created fake user again when you use
     weircd_fakeuser_Create()).*/
