//    weircdservices module
//    Copyright (C) 2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

void chanflags_list(const char* channel, const char* executinguser) {
	int id = services_chanid(channel);
	if (id < 0) {
		msguser(chanserv,executinguser,"Database error: No chan id available");return;
		return;
	}
	int k = 0;
	int r = 0;
	sqlite3_stmt *stmt = NULL;
	char statement[500];
	snprintf(statement,499,"SELECT * FROM chanflagstable WHERE chanid=%d",id);
	statement[499] = 0;
	if (sqlite3_prepare_v2(chandb,statement,-1,&stmt,0) != SQLITE_OK) {
		printf("getvalue: Could not prepare statement.\n");
		if (stmt != NULL) {sqlite3_finalize(stmt);}
		msguser(chanserv,executinguser,"Database error: Statement preparation failed");
		return;
	}
	char response[513];
	snprintf(response,512,"Flags of \002%s\002:",channel);response[512] = 0;
	msguser(chanserv,executinguser,response);
	char* ownername = services_getchannelowner(channel);
	int ownerentry = 0;
	while (sqlite3_step(stmt) == SQLITE_ROW) {
		char name[50] = "";
		char modes[50] = "";
		r = 0;
		k = sqlite3_column_count(stmt);
		while (r < k) {
			if (strcasecmp(sqlite3_column_name(stmt,r),"username") == 0) {
				const char* v = sqlite3_column_text(stmt,r);
				if (v != NULL) {
					strncpy(name,v,49);name[49] = 0;
				}
			}
			if (strcasecmp(sqlite3_column_name(stmt,r),"flags") == 0) {
				const char* v = sqlite3_column_text(stmt,r);
				if (v != NULL) {
					strncpy(modes,v,49);modes[49] = 0;
				}
			}
			r += 1;
		}
		strlwr(name);
		//now print it out
		int printedasowner = 0;
		if (ownername != NULL) {
			if (strcasecmp(ownername,name) == 0) {
				ownerentry = 1;
				printedasowner = 1;
				snprintf(response,512," - %s  Q%s",name,modes);
				response[512] = 0;
				msguser(chanserv,executinguser,response);
			}
		}
		if (printedasowner == 0) {
			snprintf(response,512," - %s  %s",name,modes);
			response[512] = 0;
			msguser(chanserv,executinguser,response);
		}
	}
	if (stmt != NULL) {sqlite3_finalize(stmt);}
	if (ownerentry == 0) {
		snprintf(response,512," - %s  Q",ownername);
		response[512] = 0;
		msguser(chanserv,executinguser,response);
	}
	msguser(chanserv,executinguser,"End of flags list.");
	free(ownername);
}
int chanflags_checkformodify(const char* user, const char* channel) {
	char response[512];
	int chanid = services_chanid(channel);
	if (chanid <= 0) {
		msguser(chanserv,user,"Database error: Cannot obtain chan id");
		return -1;
	}
	char* owner = services_getchannelowner(channel);
	if (owner == NULL) {
		msguser(chanserv,user,"Database error: Cannot obtain owner name");
		return -1;
	}
	if (strcasecmp(owner,user) != 0) {
		free(owner);
		const char* executorflags = services_getchanflags(user,chanid);
		if (executorflags == NULL) {
			msguser(chanserv,user,"Database error: Cannot obtain owner flags");
			return -1;
		}
		if (charpos(executorflags,'F') < 0) {
			snprintf(response,499,"You may not modify %s's channel list.",channel);
			response[499] = 0;
			msguser(chanserv,user,response);
			return -1;
		}
		return 0;
	}else{
		free(owner);
		return 0;
	}
}
void chanflags_change(const char* channel, const char* executinguser, const char* targetuser, const char* flags, int delete) {
	char unknownflags[50] = "";
	char unchangeableflags[50] = "";
	char newflags[50] = "";
	char flagsbuf[50] = "";
	char response[451];
	int isowner = 0;
	char* getowner = services_getchannelowner(channel);
	if (getowner != NULL) {
		if (strcasecmp(getowner,targetuser) == 0) {isowner = 1;}
		free(getowner);
	}
	int chanid = services_chanid(channel);
	if (chanid <= 0) {
		msguser(chanserv,executinguser,"Database error: Cannot obtain chan id");
		return;
	}
	const char* currentflags = services_getchanflags(targetuser,chanid);
	if (currentflags == NULL) {
		msguser(chanserv,executinguser,"Database error: Cannot obtain flags information");
		return;
	}
	strncpy(newflags,currentflags,49);newflags[49] = 0;
	int changes = 0;
	int r = 0;
	while (r < strlen(flags)) {
		if (flags[r] == 'Q') {
			if (strlen(unchangeableflags) < 49 && charpos(unchangeableflags,flags[r]) < 0) {
				unchangeableflags[strlen(unchangeableflags)+1] = 0;
				unchangeableflags[strlen(unchangeableflags)] = flags[r];
			}
			r++;continue;
		}
		if (flags[r] > 127 || flags[r] < 32) {r++;continue;}
		if (flags[r] == 'O' || flags[r] == 'o' || flags[r] == 'V' || flags[r] == 'v' || flags[r] == 'H' || flags[r] == 'I' || flags[r] == 'h' || flags[r] == 'F') {
			if (delete == 0) {
				if (charpos(newflags,flags[r]) < 0) {
					newflags[strlen(newflags)+1] = 0;
					newflags[strlen(newflags)] = flags[r];
					changes = 1;
				}
			}else{
				strcpy(flagsbuf,"");
				int count = 0;
				while (count < strlen(newflags)) {
					if (newflags[count] != flags[r]) {
						flagsbuf[strlen(flagsbuf)+1] = 0;
						flagsbuf[strlen(flagsbuf)] = newflags[count];
					}else{
						changes = 1;
					}
					count++;
				}
				strcpy(newflags,flagsbuf);
			}
			r++;continue;
		}else{
			if (strlen(unknownflags) < 49 && charpos(unknownflags,flags[r]) < 0) {
				unknownflags[strlen(unknownflags)+1] = 0;
				unknownflags[strlen(unknownflags)] = flags[r];
			}
			r++;continue;
		}
	}
	if (strlen(unchangeableflags) > 0) {
		snprintf(response,450,"Those flags cannot be manipulated by you: \002%s\002",unchangeableflags);
		response[449] = 0;
		msguser(chanserv,executinguser,response);
	}
	if (strlen(unknownflags) > 0) {
		snprintf(response,450,"Those flags do not exist: \002%s\002",unknownflags);
		response[449] = 0;
		msguser(chanserv,executinguser,response);
	}
	if (changes == 1) {
		if (services_setchanflags(targetuser,chanid,newflags) < 0) {
			msguser(chanserv,executinguser,"Database error.");
			return;
		}
	}
	if (changes == 1 || (strlen(unknownflags) <= 0 && strlen(unchangeableflags) <= 0)) {
		if (isowner == 0) {
			snprintf(response,450,"Flags for \002%s\002 on %s are now: \002%s\002",targetuser,channel,newflags);
		}else{
			snprintf(response,450,"Flags for \002%s\002 on %s are now: \002Q%s\002",targetuser,channel,newflags);
		}
		response[449] = 0;
		msguser(chanserv,executinguser,response);
	}
}
void chanflags_add(const char* channel, const char* executinguser, const char* targetuser, const char* flags) {
	chanflags_change(channel,executinguser,targetuser,flags,0);
}
void chanflags_del(const char* channel, const char* executinguser, const char* targetuser, const char* flags) {
	chanflags_change(channel,executinguser,targetuser,flags,1);
}
void chanflags_erase(const char* channel, const char* executinguser, const char* targetuser) {
	char response[451];
	int chanid = services_chanid(channel);
	if (chanid <= 0) {
		msguser(chanserv,executinguser,"Database error: Cannot obtain chan id");
		return;
	}
	if (services_setchanflags(targetuser,chanid,NULL) < 0) {
		msguser(chanserv,executinguser,"Database error.");
		return;
	}
	snprintf(response,450,"Flags for \002%s\002 on %s are now \002erased\002.",targetuser,channel);
	response[449] = 0;
	msguser(chanserv,executinguser,response);
}