//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

struct idstorage {
	int id;
	void* ptr;
};
void* idstorages = NULL;

/* some function pointers */
typedef void (*defaultfunc)(void);
typedef int (*startpassivefunc)(const char*, int, int, int);

void linkingapi_WipeStorage(int id) {
	void* item = idstorages;
	while (item != NULL) {
		struct idstorage* i = linkedlist_GetData(item);
		if (i->id == id) {
			linkedlist_RemoveItem(&idstorages,item);
			return;
		}
		item = linkedlist_NextItem(item);
	}
}

int linkingapi_StartPassiveLink(const char* modname, const char* interface, int port, int useipv6, int id) {
	module_Switched(modname);module_SetArea(MODAREA_GENERICFUNC);
	startpassivefunc func = modman_GetData(modname,"module_PassiveLinkAt");
	struct idstorage i;memset(&i,0,sizeof(i));i.id = id;
	linkedlist_Append(&idstorages,&i,sizeof(i));
	int retval = func(interface, port, useipv6, id);
	if (retval < 0) {
		linkingapi_WipeStorage(id);
		return -1;
	}
	return 0;
}
void* linkingapi_StorageByHandle(int handle) {
	void* item = idstorages;
	while (item != NULL) {
		struct idstorage* i = linkedlist_GetData(item);
		if (i->id == handle) {
			return item;
		}
		item = linkedlist_NextItem(item);
	}
	return NULL;
}
void weircd_linking_SetConnectionDataPointer(int connectionhandle, void* ptr) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	void* item = linkingapi_StorageByHandle(connectionhandle);
	if (item == NULL) {return;}
	struct idstorage* i = linkedlist_GetData(item);
	i->ptr = ptr;
}
void* weircd_linking_GetConnectionDataPointer(int connectionhandle) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return NULL;	
	}
	void* item = linkingapi_StorageByHandle(connectionhandle);
	if (item == NULL) {return NULL;}
	struct idstorage* i = linkedlist_GetData(item);
	return i->ptr;
}
void weircd_linking_CycleConnections_Start() {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	links_StartIDCycle(module_Current());
}
int weircd_linking_CycleConnections_Do() {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	return links_DoIDCycle();
}
int weircd_sockets_GetIPLen() {
	return so_GetIPLen();
}
int weircd_linking_CheckPassword(int connectionhandle, const char* password, char** sendpassword) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	/* ok. there are two possibilities. given our link is active, the thing is easy. find the matching
	   active link: */
	// (1) active links
	printf("searching password match for '%s' (%d)\n",password,connectionhandle);
	void* item = listoflinks;
	while (item != NULL) {
		struct weircdlink* wl = linkedlist_GetData(item);
		if ( wl->activeconnectionid == connectionhandle && wl->type == LINKTYPE_ACTIVE) {
			printf("found active link match\n");
			if (strcasecmp(wl->recvpassword, password) == 0) {
				*sendpassword = wl->sendpassword;
				return 0;
			}else{
				return -1;
			}
		}
		item = linkedlist_NextItem(item);
	}
	/* seems we need to search a passive link */
	// (2) passive links
	void* passivelinkptr = NULL;
	item = listoflinks;
	while (item != NULL) {
		struct weircdlink* wl = linkedlist_GetData(item);
		if (wl->activeconnectionid == connectionhandle && wl->type == LINKTYPE_PASSIVE) {
			passivelinkptr = item;
			break;
		}
		item = linkedlist_NextItem(item);
	}
	if (passivelinkptr == NULL) {return -1;} //no match for our connection handle.
	// (3) active link assigned to passive link
	item = listoflinks;
	while (item != NULL) {
		struct weircdlink* wl = linkedlist_GetData(item);
		if (wl->dependson == passivelinkptr && wl->type == LINKTYPE_ACTIVE) {
			printf("found depending active link\n");
			//ok, that active link is assigned to our passive link. check password
			if (strcasecmp(wl->recvpassword, password) == 0) {
				*sendpassword = wl->sendpassword;
				return 0;
			}
			// otherwise, do NOT cancel but continue looping. the password may match
			// *any* of the associated active links, so we need to test all of them
		}
		item = linkedlist_NextItem(item);
	}
	// no active link that is assigned to the passive link had a matching password.
	return -1;
}
int weircd_linking_IsServerPresent(const char* name) {
	if (server_GetByName(name) != NOSERVER) {
		return 0;
	}
	return -1;
}
void weircd_linking_IntroduceUser(const char* name, int hopcount, unsigned long lastnickchange, const char* userid,
const char* host,
const char* ip, const char* uid, const char* realname, const char* servername, const char* usermodes) {
	SERVER s = server_GetByName(servername);
	if (s == NOSERVER) {return;}
	USER u = server_AddUser(s, name, userid, usermodes);
	if (u == NOUSER) {return;}
	if (uid != NULL) {
		if (udat(u)->uid != NULL) {free(udat(u)->uid);}
		udat(u)->uid = malloc(strlen(uid)+1);
		if (udat(u)->uid != NULL) {
			strcpy(udat(u)->uid,uid);
		}
	}
	udat(u)->lastnickchange = lastnickchange;
	if (lastnickchange == 0) {
		udat(u)->lastnickchange = ts();
	}
	if (host != NULL) {
		strcopy(udat(u)->host, host, COMPILEOPTION_MAXHOSTLEN-1);
	}
	if (ip != NULL) {
		strcopy(udat(u)->ip,ip,IPMAXLEN);
	}
	if (realname != NULL) {
		strcopy(udat(u)->realname, realname, COMPILEOPTION_MAXUSERIDLEN);
	}
}
void weircd_linking_RaiseEvent(const char* servername, int eventid, int eventparametercount, ...) {
	
}
void weircd_linking_CollideUser(const char* name) {
	USER u = usr_FindByNick(name);
	if (u == NOUSER) {return;}
	userQuit(u,USERQUIT_COLLISION);
}
int weircd_linking_IntroduceServer(const char* name, const char* sid, const char* description, const char* parentname) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	SERVER parent = NOSERVER;
	/* check whether a parent server is given and whether we know of it */
	if (parentname != NULL) {
		parent = server_GetByName(parentname);
		if (parent == NOSERVER) {
			/* we never heard of this server. */
			return -1;
		}
	}
	/* check whether the server is already known to us: */
	if (sid != NULL) {
		if (server_GetByName(sid) != NOSERVER) {
			return -1;
		}
	}else{
		if (server_GetByName(name) != NOSERVER) {
			return -1;
		}
	}
	/* ok. add new server */
	server_Add(name, sid, description, parent);
	return 0;
}
void weircd_linking_SplitServer(const char* name) {
	/* do not restrict this function. when the module is broken, it shall still be able
	   to somehow remove its servers. maybe we can prevent a huge trash-o-matic then */
	SERVER tg = NOSERVER;
	tg = server_GetByName(name);
	if (tg == NOSERVER) {
		return;
	}
	server_Remove(tg); //omg, now it's gone!!
}
char sidbuf[4];
const char* weircd_linking_GetSID() {
	strcopy(sidbuf,config_StrVal("SID"),3);
	if (strlen(sidbuf) < 3) {return NULL;}
	return sidbuf;
}
void linkmsg(const char* msg) {
	printf("%s\n",msg);
	IOnotice(msg);
}
char linkmsgbuf[510];
void weircd_linking_AnnounceEstablishedLink(const char* target) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	snprintf(linkmsgbuf,509,"[LINKS] Link established to \"%s\".",target);
	linkmsg(linkmsgbuf);
}
void weircd_linking_AnnounceBrokenLink(const char* target, const char* reason) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	snprintf(linkmsgbuf,509,"[LINKS] Disconnected from \"%s\": %s",target,reason);
	linkmsg(linkmsgbuf);
}
void weircd_linking_AnnounceLinkConnectFail(const char* target, const char* reason) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	snprintf(linkmsgbuf,509,"[LINKS] Failed to establish link to \"%s\": %s",target,reason);
	linkmsg(linkmsgbuf);
}
void weircd_linking_AnnounceLinkConnectAttempt(const char* target) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	snprintf(linkmsgbuf,509,"[LINKS] Attempting to establish a link to \"%s\"...",target);
	linkmsg(linkmsgbuf);
}
const char* weircd_linking_GetUID(const char* user) {
        USER u = usr_FindByNick(user);
        if (u == NOUSER) {return NULL;}
        uids_Activate();
        return udat(u)->uid;
}
const char* weircd_linking_NickByUID(const char* uid) {
        usr_CycleInit();
        USER u = usr_DoCycle();
        while (u != NOUSER) {
                if (udat(u)->pleaseremove == 0) {
                        if (udat(u)->uid != NULL) {
                                if (strcasecmp(udat(u)->uid, uid) == 0) {
                                        return udat(u)->nickname;
                                }
                        }
                }
                u = usr_DoCycle();
        }
        return NOSERVER;
}