//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#if defined(_WIN32) || defined(_WIN64) || defined(WIN32) || defined(WIN64) || defined(__WIN32__) || defined(WINDOWS)
#define WIN
#endif

#ifndef WIN

#include <sys/types.h>
#include <sys/select.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#if (HAVE_MALLOC_H != 0)
#include <malloc.h>
#endif
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#endif

#ifdef WIN

#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <malloc.h>

#endif

#include "sockets.h"


int wantsend = 0; //send counter, if <= zero we don't want to send anything


fd_set weircdselectset;
fd_set* readset = NULL;
fd_set* writeset = NULL;
int fdnr = 0;

int so_sysinited = 0;
static int soinitfunc() {
	if (so_sysinited == 1) {return 0;}
	FD_ZERO(&weircdselectset);
#ifdef WIN
	WSADATA wsa;
	long r = WSAStartup(MAKEWORD(2,0),&wsa);
	if (r != 0) {
		return -1;
	}
#endif
	if (readset == NULL) {
		readset = malloc(sizeof(weircdselectset));
		if (readset == NULL) {
			return -1;
		}
	}
	if (writeset == NULL) {
		writeset = malloc(sizeof(weircdselectset));
		if (writeset == NULL) {
			return -1;
		}
	}
	FD_ZERO(writeset);FD_ZERO(readset);
	so_sysinited = 1;
	return 0;
}
int so_Startup() {
	return soinitfunc();
}
void so_SetSocketNonblocking(int socket) {
	if (so_sysinited == 0) {return;}
#ifndef WIN
	int save_fd;
	save_fd = fcntl(socket, F_GETFL);
	save_fd |= O_NONBLOCK;
	fcntl(socket, F_SETFL, save_fd);
#endif
#ifdef WIN
	long l = 1;
	ioctlsocket(socket,FIONBIO,(unsigned long*)&l);
#endif
}
void so_ManageSocketWithSelect(int socket) {
	if (so_sysinited == 0) {return;}
	so_SetSocketNonblocking(socket);
	FD_SET(socket,&weircdselectset);
	if (fdnr <= socket) {fdnr = socket+1;}
	return;
}
int so_CreateSocket(int addToSelect, int iptype) {
	if (so_sysinited == 0) {return -1;}
	int newsock = -1;
	if (iptype == IPTYPE_IPV6) {
#ifdef IPV6
		newsock = socket(AF_INET6, SOCK_STREAM, 0);
#endif	
	}
	if (iptype == IPTYPE_IPV4) {
		newsock = socket(AF_INET, SOCK_STREAM, 0);
	}
	if (newsock < 0) {return -1;}
	so_SetSocketNonblocking(newsock);
	if (addToSelect == 1) {so_ManageSocketWithSelect(newsock);}
	return newsock;
}
int so_MakeSocketListen(int socket, int port, int iptype, const char* bindip) {
	if (so_sysinited == 0) {return -1;}
	// ( 1 ) --- prepare address struct
#ifdef IPV6
	struct sockaddr_in6 addressstruct6;
#endif
	struct sockaddr_in addressstruct4;
	if (iptype == IPTYPE_IPV6) {
#ifdef IPV6
		memset(&addressstruct6, 0, sizeof(addressstruct6));
		addressstruct6.sin6_family = AF_INET6;
#else
		return  -1;
#endif
	}else{
		memset(&addressstruct4, 0, sizeof(addressstruct4));
		addressstruct4.sin_family = AF_INET;
	}
	// ( 2 ) --- get the ip specified in "bindip" into the address struct
	if (strcasecmp(bindip,"any") != 0) {
		//1- now bind depending on operating system and ip family (ipv4 or ipv6)
		/* LINUX/UNIX */
#ifndef WIN
		if (iptype == IPTYPE_IPV6) {
		#ifdef IPV6
			inet_pton(AF_INET6, bindip, &(addressstruct6.sin6_addr));
		#else
			return -1;
		#endif
		}else{
			inet_pton(AF_INET, bindip, &(addressstruct4.sin_addr.s_addr));
		}
#endif
		/* WINDOWS */
#ifdef WIN
		if (iptype == IPTYPE_IPV6) {
		#ifdef IPV6
			DWORD size = sizeof(addressstruct6.sin6_addr);
			if (WSAStringToAddress(bindip, AF_INET6, NULL, &(addressstruct6.sin6_addr), &size) != 0) {
				return -1;
			}
		#else
			return -1;
		#endif
		}else{
			addressstruct4.sin_addr.s_addr = inet_addr(bindip);
		}
#endif

		//2- check if binding worked
		
		if (iptype == IPTYPE_IPV6) {
#ifdef IPV6
			if (IN6_IS_ADDR_UNSPECIFIED(&addressstruct6.sin6_addr) == 1) {
				return -1;
			}
#else
			return -1;
#endif
		}else{
			if (addressstruct4.sin_addr.s_addr == INADDR_ANY || addressstruct4.sin_addr.s_addr == INADDR_NONE) {
				return -1;
			}
		}
	}else{
		if (iptype == IPTYPE_IPV6) {
#ifdef IPV6
			addressstruct6.sin6_addr = in6addr_any;
#else
			return -1;
#endif
		}else{
			addressstruct4.sin_addr.s_addr = INADDR_ANY;
		}
	}
	// ( 3 ) --- set port
	if (iptype == IPTYPE_IPV6) {
#ifdef IPV6
		addressstruct6.sin6_port = htons(port);
#else
		return -1;
#endif
	}else{
		addressstruct4.sin_port = htons(port);
	}
	// allow reusing the address on unix systems
#ifndef WIN
	long lclyes = 1;
	setsockopt(socket, SOL_SOCKET,SO_REUSEADDR, &lclyes, sizeof(int));
#endif
	// ( 4 ) --- the actual binding process & listening
	if (iptype == IPTYPE_IPV6) {
#ifdef IPV6
		if (bind(socket,(struct sockaddr *) &addressstruct6,sizeof(addressstruct6)) < 0) {
			return -1;
		}
#else
		return -1;
#endif
	}else{
		if (bind(socket,(struct sockaddr *) &addressstruct4,sizeof(addressstruct4)) < 0) {
			return -1;
		}
	}
	listen(socket, port);
	return 0;
}
/*const char* ResolveAddress(const char* tg) {
	
}*/
int so_ConnectSocketToIP(int socket, const char* ip) {
	if (so_sysinited == 0) {return -1;}
	//FIXME: implement this?
	return -1;
}
int so_SelectWait(int maximumsleeptime) {
	if (so_sysinited == 0) {return 0;}
	struct timeval t;
	memset(&t,0,sizeof(struct timeval));
	t.tv_usec = maximumsleeptime*1000;
	memcpy(readset,&weircdselectset,sizeof(weircdselectset));
	memcpy(writeset,&weircdselectset,sizeof(weircdselectset));
	int r;
	//printf("DEBUG: select(): Call... (wantsend: %d)\n",wantsend);
	if (wantsend > 0) {
		r = select(fdnr,readset,writeset,NULL,&t);
	}else{
		r = select(fdnr,readset,NULL,NULL,&t);
		FD_ZERO(writeset);
	}
	if (r <= 0) {
		FD_ZERO(readset);FD_ZERO(writeset);
	}
	return r;
}
void so_CloseSocket(int socket) {
	if (so_sysinited == 0) {return;}
	FD_CLR(socket, &weircdselectset);
	if (readset != NULL) {FD_CLR(socket, readset);}
	if (writeset != NULL) {FD_CLR(socket, writeset);}
	close(socket);
}
int so_AcceptConnection(int socket, int iptype, char* writeip, int* writesocket) {
	if (so_sysinited == 0) {return -1;}
	socklen_t addrlen;
	
#ifdef IPV6
	struct sockaddr_in6 address6;
#endif
	struct sockaddr_in address4;
	if (iptype == IPTYPE_IPV6) {
		#ifdef ipv6
		addrlen = sizeof(address6);
		#else
		return -1;
		#endif
	}else{
		addrlen = sizeof(address4);
	}
	int new_socket;
	if (!FD_ISSET(socket,readset)) {return -1;}
	if (iptype == IPTYPE_IPV6) {
		#ifdef ipv6
		new_socket = accept(socket,(struct sockaddr *) &address6,&addrlen);
		#else
		return -1;
		#endif
	}else{
		new_socket = accept(socket,(struct sockaddr *) &address4,&addrlen);
	}
	if (new_socket > 0) {
		so_ManageSocketWithSelect(new_socket);
		//write the ip address into the "writeip" buf
		if (writeip != NULL) {
			int cnt = IPMAXLEN+1;
			if (iptype == IPTYPE_IPV4) {
				cnt = IPV4LEN+1;
			}
#ifdef WIN // WINDOZE
			if (iptype == IPTYPE_IPV6) {
				#ifdef IPV6
				getnameinfo((struct sockaddr *)&address6, sizeof(struct
				sockaddr_in6), writeip, cnt, NULL, 0, NI_NUMERICHOST);
				writeip[IPMAXLEN] = 0;
				#else
				return -1;
				#endif
			}else{
				getnameinfo((struct sockaddr *)&address4, sizeof(struct
				sockaddr_in), writeip, cnt, NULL, 0, NI_NUMERICHOST);
				writeip[IPV4LEN] = 0;
			}
#endif

#ifndef WIN // LINUX/POSIX
			if (iptype == IPTYPE_IPV6) {
				#ifdef IPV6
				inet_ntop(AF_INET6, &address6.sin6_addr, writeip, cnt);
				writeip[IPMAXLEN] = 0;
				#else
				return -1;
				#endif
			}else{
				inet_ntop(AF_INET, &address4.sin_addr, writeip, cnt);
				writeip[IPV4LEN] = 0;
			}
#endif
			
		}
		if (writesocket != NULL) {
			*writesocket = new_socket;
		}
		return 0;
	}
	return -1;
}
int so_GetIPLen() {
	return IPMAXLEN;
}
int so_CheckIp(const char* ip) { // check if an IP is an IP
	return -1;
}
int so_SelectSaysWrite(int sock) {
	if (so_sysinited == 0) {return -1;}
	if (!FD_ISSET(sock, writeset)) {
		return -1;
	}
	return 0;
}
int so_SelectSaysRead(int sock) {
	if (so_sysinited == 0) {return -1;}
	if (!FD_ISSET(sock, readset)) {
		return -1;
	}
	return 0;
}
int so_SendOnSocket(int sock,const char* buffer,int buffsize) {
	if (so_sysinited == 0) {return -1;}
	errno = 0;
	int retval = send(sock,buffer,buffsize,0);//MSG_NOSIGNAL);
	if (retval > 0) {
		return retval;
	}else{
#ifdef WIN
		int err = WSAGetLastError();
		if ((err != WSAEWOULDBLOCK && err != WSAEINTR && err != WSAENOBUFS) || retval == 0) {
#endif
#ifndef WIN
		if ((errno != EWOULDBLOCK && errno != EINTR && errno != ENOBUFS) || retval == 0) {
#endif
			return 0;//EOF/fatal error which leads to stream stopping
		}else{
			return -1;//just a temporary error
		}
	}
}
int so_ReceiveData(int socket, char* buf, int len) {
	if (so_sysinited == 0) {return -1;}
	int z = recv(socket,buf,len,0);
	if (z > 0) {//we received something!
		return z;
	}else{	//something happened. check if it's no EAGAIN/-WOULDBLOCK (and other temporary stuff)
#ifdef WIN
		int err = WSAGetLastError();
		if ((err != WSAEWOULDBLOCK && err != WSAEINTR && err != WSAENOBUFS) || z == 0) {
#endif
#ifndef WIN
		if ((errno != EWOULDBLOCK && errno != EINTR && errno != EAGAIN && errno != ENOBUFS) || z == 0) {
#endif
			return 0;//EOF/fatal error which leads to stream stopping
		}else{
			return -1;//just a temporary error
		}
	}
}