//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

// The user manager manages the socket listeners for the user connections and
// obtains the data from the connections, proceeding it line for line to irc.c

#include "weircd.h"

struct socketinfo {
	char ip[IPMAXLEN+1];
	int socket;
	int port;
	int iptype;
};
void* socketlist = NULL;

int userman_IsSockInConf(int port, const char* ip) {
	void* item = conflisteners;
	while (item != NULL) {
		struct conflistener* c = linkedlist_GetData(item);
		if (strcasecmp(c->ip,ip) == 0 && c->port == port) {
			return 0;
		}
		item = linkedlist_NextItem(item);
	}
	return -1;
}
int userman_IsPortOpened(int port, const char* ip) {
	void* item = socketlist;
	while (item != NULL) {
		struct socketinfo* sinfo = linkedlist_GetData(item);
		if (sinfo != NULL) {
			if (sinfo->port == port && strcasecmp(sinfo->ip,ip) == 0) {
				return 0;
			}
		}
		item = linkedlist_NextItem(item);
	}
	return -1;
}
void userman_CloseListener(int sock) {
	
}
int userman_AddListener(int port, const char* ip, int iptype) {
	int newsock = so_CreateSocket(1, iptype);
	if (newsock < 0) {
		return -1;
	}
	if (so_MakeSocketListen(newsock,port,iptype, ip) < 0) {
		so_CloseSocket(newsock);
		return -1;
	}
	struct socketinfo sinfo;
	memset(&sinfo,0,sizeof(sinfo));
	sinfo.port = port;
	sinfo.socket = newsock;
	sinfo.iptype = iptype;
	strcopy(sinfo.ip,ip,IPMAXLEN);
	void* item = linkedlist_Append(&socketlist,&sinfo,sizeof(sinfo));
	if (item == NULL) {
		so_CloseSocket(newsock);
		return -2;
	}
	return 0;
}
void userman_CloseOldListeners() {
	void* item = socketlist;
	while (item != NULL) {
		struct socketinfo* sinfo = linkedlist_GetData(item);
		if (sinfo != NULL) {
			if (userman_IsSockInConf(sinfo->port,sinfo->ip) != 0) {
				userman_CloseListener(sinfo->socket); /* close listener */
				linkedlist_RemoveItem(&socketlist,item); /* remove socket info list entry */
			}
		}
		item = linkedlist_NextItem(item);
	}
}
int userman_OpenNewListeners() {
	void* item = conflisteners;
	while (item != NULL) {
		struct conflistener* c = linkedlist_GetData(item);
		if (c != NULL) {
			if (userman_IsPortOpened(c->port,c->ip) != 0) {
				int r = userman_AddListener(c->port,c->ip,c->iptype);
				if (r < 0) {return r;}
			}
		}
		item = linkedlist_NextItem(item);
	}
	return 0;
}
int userman_RefreshState() {
	userman_CloseOldListeners();
	int r = userman_OpenNewListeners();
	if (r < 0) {
		return r;
	}
	return 0;
}
void userman_CloseListenersForShutdown() {
	void* item = socketlist;
	while (item != NULL) {
		struct socketinfo* sinfo = linkedlist_GetData(item);
		if (sinfo != NULL) {
			close(sinfo->socket);
		}
		item = linkedlist_NextItem(item);
	}
}
void userman_CheckConnections() {
	void* item = socketlist;
	while (item != NULL) {
		struct socketinfo* sinfo = linkedlist_GetData(item);
		if (sinfo != NULL) {
			char ipbuf[IPMAXLEN+1];
			int sock;
			if (so_AcceptConnection(sinfo->socket, sinfo->iptype, ipbuf, &sock) == 0) {
				//new client! yay!
				USER newuser = usr_AddUser(sock,sinfo->socket,ipbuf,sinfo->iptype);
				if (newuser == NOUSER) {
					so_CloseSocket(sock);
				}else{
					//int id = usr_SlotCount();
					so_ManageSocketWithSelect(sock);
					irc_SendStartupNotice(newuser);
					//udat(newuser)->debugid = id;
				}
			}else{
				//printf("no new client!\n");
			}
		}
		item = linkedlist_NextItem(item);
	}
}
void userman_SplitCMD(USER userid, const char* cmd) {
	char lclstring[600];
	char lclstring2[600];
	strncpy(lclstring,cmd,599);lclstring[599] = 0;
	trimleft(lclstring);
	strcpy(lclstring2,lclstring);
	if (charpos(lclstring2,32) >= 0) {
		cut(lclstring2,charpos(lclstring2,32)+1);
	}else{
		lclstring2[0] = 0;
	}
	if (charpos(lclstring,32) >= 0) {
		lclstring[charpos(lclstring,32)] = 0;
	}
	irc_UserCMD(userid,lclstring,lclstring2);
}
void userman_ExtractCMDs(USER userid) {
	char lclstring[600];
	strncpy(lclstring,udat(userid)->recvbuffer,599);
	lclstring[599] = 0;
	int r = 0;
	r = charpos(udat(userid)->recvbuffer,'\n');
	if (r >= 0) {
		cut(udat(userid)->recvbuffer,r + 1);
		if (r <= 599) {
			lclstring[r] = 0;
		}
	}
	udat(userid)->floodprotect.floodpoints += 1;
	trimleft(lclstring);
	if (strlen(lclstring) <= 0) {return;}
	userman_SplitCMD(userid,lclstring);
}
void userman_ProceedReceiving() {
	char lclbuffa[USER_RECVBUFFERSIZE + 1];
	usr_CycleInit();
	USER u = usr_DoCycle();
	while (u != NOUSER) {
		if (udat(u)->socket >= 0 && udat(u)->pleaseremove == 0 && udat(u)->hopcount <= 0 && udat(u)->fakeuser == 0) {
			if (so_SelectSaysRead(udat(u)->socket) != 0) {
				u = usr_DoCycle();continue;
			}
			if (strlen(udat(u)->recvbuffer) < USER_RECVBUFFERSIZE - 1) {//check if our buffer is not full
				//calculate the len for receiving (k) and do receive
				int k = strlen(udat(u)->recvbuffer);
				errno = 0;
				int z = so_ReceiveData(udat(u)->socket,lclbuffa,USER_RECVBUFFERSIZE - (k+1));
				//now check z for result
				if (z > 0) {//we received something!
					lclbuffa[z] = 0;
					strcat(udat(u)->recvbuffer,lclbuffa);
					k = k+z;
					z = 0;
					while (z < k) {
						if (udat(u)->recvbuffer[z] == 0) {udat(u)->recvbuffer[z] = 32;}
						z += 1;
					}
					fixlinebreaks(udat(u)->recvbuffer,strlen(udat(u)->recvbuffer));
					while (charpos(udat(u)->recvbuffer,'\n') >= 0) {
						userman_ExtractCMDs(u);
					}
				}else{	//something happened. check if it's no EAGAIN/-WOULDBLOCK (and other temporary stuff) and if it's not -> quit! (as the connection will be gone)
					if (z == 0) {
						so_CloseSocket(udat(u)->socket);
						udat(u)->socket = -1;
						userQuit(u,USERQUIT_EOF);
					}
				}
			}else{	//BUFFER IS FULL - this means the client cheated us with >512 bytes lines which is non-rfc compliant --> throw it away!
				strcpy(udat(u)->recvbuffer,"");
				udat(u)->floodprotect.floodpoints += 4;
			}
		}
		u = usr_DoCycle();
	}
}
void userman_ProceedSending() {
	usersend_Proceed();
}