//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2010 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#define USERQUIT_SIGPIPE 1
#define USERQUIT_QUIT 2
#define USERQUIT_EOF 3
#define USERQUIT_TIMEOUT 4
#define USERQUIT_ERROR 5
#define USERQUIT_SENDQEXCEEDED 6
#define USERQUIT_EXCESSFLOOD 7
#define USERQUIT_REGISTERTIMEOUT 8
#define USERQUIT_NETSPLIT 9
#define USERQUIT_LINKCLOSE 10
#define USERQUIT_COLLISION 11
#define USERQUIT_CUSTOMREMOTEQUIT 12
#define USER_MAXSENDBUFFERS 20
#define USER_RECVBUFFERSIZE 1024
struct weircd_user_floodprotect {
	int floodpoints;
	unsigned long long lastlist;
	unsigned long long lastmotd;
};
struct weircd_user_chaninfo {
	CHANNEL cptr;
	int iamop;
	int iamhalfop;
	int iamvoice;
	int restrictedcache; //remember if a user is restricted: -1 don't know (needs to be found out yet and cached), 0 no, 1 yes
};
struct weircd_user {
	//authentication info
	int authed;//001 message passed
	long long authtime;
	long authpongvalue;
	int passwordchecked;
	//send block
	int sendblocks;//sendblocks of this user still waiting to be sent
	char* sendbuffer[USER_MAXSENDBUFFERS];
	//receive buffer
	char recvbuffer[USER_RECVBUFFERSIZE];
	//irc op info
	int isircop;
	struct ircopflags opflags; //defined in ircopstruct.h
	//server info (non local users)
	int hopcount;
	SERVER homeserver;
	char* servname;
	//displayed accountname (can be set by services)
	char* accountname;
	//modes
	#ifndef nowimode
	int mode_invisible;
	int mode_walluser;
	#endif
	int mode_bot;
	int mode_connectionnotices; // can be enabled to receive notices about connecting users
	int mode_servicesbot; // does the user have services rights (+S)
	int mode_noflood; // turns off excess flood but not sendq exceeded (+F)
	int mode_fixedhost; // (+h) tells services about custom irc op host so it doesn't get changed on services login/logout
	//away string or NULL
	char* awayreason;
	//remove info
	int pleaseremove;
	long long pleaseremove_time;
	int sendbeforeremove;
	char* quitreason;
	//connection info
	int socket;
	char ip[IPMAXLEN+1];
	int iptype;
	int mastersocketid;//to close the user down if listener port they connected to is closed
	long CONNECTtime;
	//PING info
	long long ping_lastping;
	long long ping_waitingforpong;
	long ping_pingval;
	//flood info
	struct weircd_user_floodprotect floodprotect;
	long long lastnickchange;
	//channel info
	void* channelinfolist;
	//nick, userid, realname & host
	char nickname[COMPILEOPTION_MAXNICKLEN + 1];
	char suserid[COMPILEOPTION_MAXUSERIDLEN + 1];
	char realname[COMPILEOPTION_MAXREALNAMELEN + 1];
	char host[COMPILEOPTION_MAXHOSTLEN];
	//fake user flag
	int fakeuser; // a fake user which has no real tcp connection but is used for services or similar
	//user mark
	int marked; // a little flag for use in some complex user handling functions
	//a 5 char letter-based unique UID. used for TS6 linking
	char* uid;
	//debug id
	int debugid;
	//is it a web cgi registered user?
	int webirc;
};



#define NOUSER NULL

// 1 -- User Array Management

USER usr_FindByNick(const char* nick);
USER usr_AddUser(int socket, int mastersocket, const char* ip, int iptype);
void usr_DelUser(USER usr);

struct weircd_user* udat(USER usr); //get the data of a user

void usr_CycleInit();
USER usr_DoCycle();// cycle through all users to do something with them

int usr_SlotCount();

void usr_CloseAllConnectionsForShutdown();

// 2 - handling the channel list of a user

void usr_chans_CycleInit(USER uid);
void* usr_chans_DoCycle();
void usr_chans_Clear(USER uid);
void usr_chans_Delete(USER usr, void* item);
struct weircd_user_chaninfo* usr_chans_GetInfo(void* item);
void* usr_chans_Add(USER uid);
int usr_chans_Count(USER uid);
void* usr_chans_GetHandleByChan(USER uid, CHANNEL chan);
void usr_chans_Print(USER uid);//for debugging

// 3 - other user handling functions

void usr_SendConnectionNotice(const char* nick, const char* sid, const char* host, const char* ip);
char usr_GetPrefix(USER userid, CHANNEL vchanid);
void usr_Init();
int usr_IsUserInChan(USER userid, CHANNEL vchanid);
int usr_IsOppedOnJoin(USER uid, const char* channame);
int usr_AmIInvited(USER usr, CHANNEL chan);
void usr_RemoveInvite(USER usr, CHANNEL chan);
void usr_NoticeAllUsers(USER source, const char* stuff);
void usr_TellAllUsersInChan(const char* stuff, CHANNEL chanid);
void usr_TellAllUsersInChanExcept(const char* stuff, CHANNEL chanid, USER excludeuser);
void usr_TellAllUsersInChan_Local(const char* stuff, CHANNEL chanid);
void usr_TellAffected(const char* stuff, USER senduser);
int usr_CheckAndRemoveEmptyChans();
void usr_PingAndRemove();
int usr_ChanUserCount(CHANNEL vchanid);
void userQuit(USER uid, int reason);
void IOnotice(const char* str);

extern void* linkeduserlist;